package com.ho.test.perf.measure.commons.aop;
import javax.servlet.http.HttpServletRequest;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;

import com.ho.test.perf.measure.commons.tools.StopWatch;



/**
 * <p>Title: PerformanceMonitor</p>
 * <p>
 * 		This probe is used to measure the response time functions Spring via AOP.
 * </p>
 * 
 * @a2.Version
 */
public class PerformanceMonitor implements MethodInterceptor {

	/**
	 * Returns a number in milliseconds between 1000 (1sec) and 5000 (5sec)
	 * @return
	 */
	public static long getRandomSleepTime(){
		int lower = 1000;
		int higher = 5000;

		long random = (long)(Math.random() * (higher-lower)) + lower;
		return random;
	}

	/**
	 * @see org.aopalliance.intercept.MethodInterceptor#invoke(org.aopalliance.intercept.MethodInvocation)
	 */
	public Object invoke(MethodInvocation method) throws Throwable {
		// Generates the MethodKey of intercepted method
		String methodKey = getMethodKey(method);
		
		// Business executed BEFORE the execution of intercepted method
		StopWatch.getInstance().start(methodKey);
		
		Object output = null;
		try {
			// Execution of intercepted method
			output = method.proceed(); 
		} finally {
			// Business executed AFTER the execution of intercepted method
			StopWatch.getInstance().stop(methodKey);
		} // end-if

		return output;
	}
	
	/**
	 * Generates the MethodKey of intercepted method
	 * @return the key used to store the items in the cache.
	 */
	private String getMethodKey(MethodInvocation method) {
	    StringBuffer cacheNameBuffer = new StringBuffer();
	    //cacheNameBuffer.append(method.getThis().getClass().getName()+":"+method.getMethod().getName() + "(");
		cacheNameBuffer.append(method.getMethod().getDeclaringClass().getName() + "." + method.getMethod().getName() + "(");
	    
	    if (method.getArguments() != null && method.getArguments().length > 0) {
	      for (int a = 0; a < method.getArguments().length; a++) {
	    	  if (0!=a){
	    		  cacheNameBuffer.append(",");
	    	  }
	        if (method.getArguments()[a] == null) {
	          cacheNameBuffer.append("null");
	        }else if (method.getArguments()[a] instanceof HttpServletRequest) {
	        	HttpServletRequest req = (HttpServletRequest) method.getArguments()[a];
		        cacheNameBuffer.append(req.getMethod() + " " + req.getLocalName()+":"+req.getLocalPort()+" " + req.getRequestURI() + "?" + req.getQueryString());
	        } else {
	          cacheNameBuffer.append(method.getArguments()[a].toString().toLowerCase());
	        } // end-if
	      } // end-for
	    } // end-if
		
	    cacheNameBuffer.append(")");
	    
		return cacheNameBuffer.toString();
	}
}
