package com.ho.test.perf.measure.commons.tools;

import java.util.Calendar;

/**
 * <p>Title: StopWatchTask</p>
 * <p>
 * Measures the duration of execution of a task.
 * </p>
 * @a2.Version
 */
public class StopWatchTask {

	// -------------------------------------------------------------- CONSTANTS
	// ----------------------------------------------------- PRIVATE ATTRIBUTES
	
	/**
	 * Indentation level of the task during the displaying of report.
	 */
	private int indent = 0;
	
	/**
	 * Hour (in ms) of tasks startup.
	 */
	private long startTime = 0;
	
	/**
	 * Duration of execution of the task in ms.
	 */
	private long execTime = 0;
	
	/**
	 * Total duration (accrued) of the task in ms.
	 */
	//private long totalExecTime = 0;
	
	/**
	 * Name/Reference of task.
	 */
	private String name;
	
	/**
	 * Additional message displayed with the task in the report.
	 */
	private String log = "";
	
	/**
	 * Indicates if the task is significant or not.
	 * <br/>If the task is significant, it will be put in evidence in the report.
	 */
	private boolean significant = false;
	

	// ------------------------------------------------------------- EXCEPTIONS
	// ------------------------------------------------------- INTERNAL CLASSES
	// ----------------------------------------------------------- CONSTRUCTORS
	/**
	 * Constructor of task.
	 */
	public StopWatchTask(String name,  int indent) {
		this.name = name;
		this.indent = indent;
	}

	// --------------------------------------------------------- PUBLIC METHODS
	/**
	 * Start the task.
	 */
	public final void start() {
		startTime = Calendar.getInstance().getTimeInMillis();
	}
	
	/**
	 * Stop the task.
	 * (and update the execution duration (exectime) and the accrued total duration (totalExecTime) of the task.
	 */
	public final void stop() {
		long endTime = Calendar.getInstance().getTimeInMillis();
		execTime = endTime - startTime;
		//totalExecTime+= execTime;
	}

	public String dumpWithoutIdent() {
		return "Task "  
				+ "name='" + name + "' "  
				+ "execTime='" + execTime + "' " 
				//+ "total : " + totalExecTime 
				;
	}
	
	/**
	 * Method toString
	 * @return
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String identStr = "";
		for (int i=0; i<indent; i++) {
			identStr += "\t";
		}
		
		String output = identStr + dumpWithoutIdent();
		if (!"".equals(log)) {
			output += "\n" + identStr + log;
		}

		return output;
	}
	
	/**
	 * Add a logging message to the task.
	 * <br/>This messahe will be included in the task during thr execution report dislpay.
	 * @param message
	 */
	public final void log(String message) {
		log += message;
	}
	
	public final void setSignificant() {
		significant = true;
	}
	
	public boolean isSignificant() {
		return significant;
	}
	
	// -------------------------------------------------------- PRIVATE METHODS
	// ------------------------------------------------------ PROTECTED METHODS
	// --------------------------------------------- ADVANCED GETTERS & SETTERS
	// ------------------------------------------------------ GETTERS & SETTERS
}
